# i-bem.js: руководство пользователя

## i-bem.js: JavaScript-фреймворк для БЭМ

`i-bem.js` — специализированный JavaScript-фреймворк для веб-разработки
в рамках [БЭМ-методологии](https://ru.bem.info/method/).

`i-bem.js` позволяет:

 * разрабатывать веб-интерфейс в терминах блоков, элементов, модификаторов;
 * описывать логику работы блока в декларативном стиле — как набор состояний;
 * легко интегрировать JavaScript-код с BEMHTML- или BH-шаблонами и CSS в стиле БЭМ;
 * гибко переопределять поведение библиотечных блоков.

`i-bem.js` не предназначен для замены фреймворка общего назначения, подобного jQuery.

**Краткий обзор содержания документа**:

* [Общие сведения](#intro) о фреймворке: связь с предметной областью БЭМ, краткое описание модульной структуры
фреймворка, шаблонного проекта и инструментов для сборки кода,
написанного с использованием `i-bem.js`.
* [Привязка JS-блоков к HTML][html] — разметка JS-блоков в
HTML-коде страницы, варианты соотношения HTML-элементов и JS-блоков.
* [Декларация блока][decl] — синтаксис описания JS-блоков.
* [Передача параметров][params] — передача параметров экземпляру блока, получение доступа к параметрам блока из экземпляра.
* [Работа с DOM-деревом][dom] — API для работы с DOM-узлами блоков:
элементы, динамическое изменение DOM-дерева (AJAX), поиск DOM-узлов.
* [Состояния блока][states] — модификаторы, триггеры на смену состояний (установку модификаторов).
* [События][events] — событийная модель `i-bem.js`: DOM- и БЭМ-события, делегирование событий.
* [Инициализация][init] — инициализация и удаление экземпляров блоков, отложенная и автоматическая инициализация.
* [Взаимодействие блоков][interact] — обращение из блока к другим блокам, классам блоков.
* [Контекст][context] —  собственные и статические свойства блока.  Статические свойства BEMDOM.
* [Что дальше?][extras] — ссылки на документацию и дополнительные материалы.

<a name="intro"></a>
## Общие сведения

<a name="intro-bem"></a>
### БЭМ-методология и JavaScript

В БЭМ-методологии веб-интерфейс строится из независимых
**блоков** у которых могут быть **элементы**. И блоки,
и элементы могут иметь состояния или особенности, описываемые **модификаторами**.

Работа веб-интерфейса обеспечивается несколькими **технологиями**
(HTML, CSS, JS и т.д.). Его реализация разбита на компоненты по блокам. Блок содержит набор **файлов технологий**, составляющих аспекты его реализации:

 * `my-block.css` — внешний вид блока;
 * `my-block.bemhtml` — шаблоны для генерации HTML-представления блока;
 * `my-block.js` — **динамическое поведение** блока в браузере.

Фреймворк `i-bem.js` позволяет разложить клиентский JavaScript на компоненты в терминах БЭМ:

* **Блок** — JS-компонент, описывающий логику работы однотипных элементов интерфейса. Например, все кнопки могут быть реализованы в виде блока `button`. В этом случае, `button.css` определяет внешний вид всех кнопок, а `button.js` — логику их работы.
На каждой странице может размещаться более одного **экземпляра блока** (например, кнопки). Каждому экземпляру блока соответствует JS-объект, в памяти браузера, хранящий его состояние. JS-объект содержит ссылку на DOM-узел, к которому привязан данный экземпляр блока.
* **Элементы** — DOM-узлы, вложенные в DOM-узел блока, с атрибутом `class`, указывающим на их роль в БЭМ-предметной области (имя блока и элемента). Элементы блока доступны через [JS-API][dom] экземпляра блока.
* **Модификаторы** — предоставляют информацию о состоянии блока и его элементов. Состояние модификаторов записывается в атрибуте `class` на DOM-узлах блока и элементов. Управление модификаторами производится через [JS-API](i-bem-js-states.ru.md#mods-api) экземпляра блока.

<a name="intro-build"></a>
### Сборка

Разработка в рамках БЭМ-методологии ведется модульно — каждый блок
программируется отдельно. Финальный исходный код веб-страниц
формируется из кода отдельных блоков с помощью процедур **сборки**.

В файловой системе блок удобно представлять в виде каталога, а реализацию блока в каждой из технологий — в виде отдельного файла:

```html
    desktop.blocks/
        my-block/
            my-block.css
            my-block.js
            my-block.bemhtml
            ...

    desktop.blocks/
        other-block/
            other-block.css
            other-block.js
            other-block.bemhtml
            ...
```


Для каждой веб-страницы код использованных на ней блоков может быть собран в единые файлы:

```html
    desktop.bundles/
        index/
            index.html
            index.css
            index.js
            ...
```


Существует два инструмента, поддерживающих БЭМ-предметную область, для сборки кода результирующих веб-страниц из отдельных описаний блоков:

* [bem-tools][];
* [ENB](https://ru.bem.info/tools/bem/enb-bem/) совместно с [enb-modules](https://github.com/enb-make/enb-modules).

Оба инструмента позволяют автоматизировать создание HTML-разметки для [привязки JS-блоков][html] и [передачи параметров экземпляру блока][params].

<a name="intro-name"></a>
### Почему i-bem.js так называется

В соответствии с БЭМ-методологией, базовая JS-библиотека БЭМ-платформы изначально разрабатывалась
как особый служебный блок. Такой подход позволяет работать с базовыми библиотеками так же, как
и с обычными блоками. В частности, структурировать код в терминах элементов и модификаторов и
гибко настраивать поведение библиотеки на разных уровнях переопределения.

Служебным блокам в БЭМ было принято давать имена с префиксом `i-`. Таким образом, имя `i-bem.js`
читается как *реализация блока `i-bem` в технологии `JS`*.

<a name="intro-use"></a>
### Как использовать i-bem.js

Фреймворк `i-bem.js` входит в состав библиотеки [bem-core](https://ru.bem.info/libs/bem-core/).

Реализация `i-bem.js` состоит из двух модулей:

* **Модуль [i-bem][]**.<br/>
Базовая реализация JS-блока `i-bem`, от которой наследуются все блоки в
`i-bem.js`. Блок `i-bem` написан с расчетом на использование в любом
JS-окружении: как на клиенте, так и на сервере например, в
Node.js).
* **Модуль [i-bem__dom][]**.<br/>
Базовая реализация блока, привязанного к DOM-узлу.
Рассчитан на использование на клиенте, опирается на работу браузеров с DOM. Зависит от `jQuery`.

Зависимости:

 * jQuery (только для модуля `i-bem__dom`). При использовании `bem-core` отдельная установка
 jQuery не требуется.
 * Модульная система [ym/modules][ym]. При использовании
   [bem-tools][] с технологией `.browser.js` (и производных от нее)
   эта зависимость удовлетворяется автоматически.

Можно использовать `i-bem.js` как часть полного стека
БЭМ-инструментов. В этом случае свой проект удобно создавать на основе
шаблонного репозитория [project-stub](https://ru.bem.info/tutorials/project-stub/), в котором настроена автоматическая установка зависимых библиотек и сборка.

Если не планируется использование других технологий БЭМ-платформы, достаточно поместить код библиотеки `bem-core` в существующий проект.


[ym]: https://github.com/ymaps/modules

[bem-tools]: https://ru.bem.info/tools/bem/

[i-bem]: https://ru.bem.info/libs/bem-core/current/desktop/i-bem/jsdoc/

[i-bem__dom]: https://ru.bem.info/libs/bem-core/current/desktop/i-bem/jsdoc/

[html]: ./i-bem-js-html-binding.ru.md

[decl]: ./i-bem-js-decl.ru.md

[dom]: ./i-bem-js-dom.ru.md

[context]: ./i-bem-js-context.ru.md

[states]: ./i-bem-js-states.ru.md

[events]: ./i-bem-js-events.ru.md

[params]: ./i-bem-js-params.ru.md

[init]: ./i-bem-js-init.ru.md

[init-live]: ./i-bem-js-init.ru.md#init-live

[interact]: ./i-bem-js-interact.ru.md

[extras]: ./i-bem-js-extras.ru.md
